@@ -100,7 +100,7 @@
 
     mtopoint = [path firstPoint];
     ponpoint = [path pointOnPoint: self];
-    if(ponpoint && [(GRPathEditor *)[path editor] isdone] && (self == mtopoint))
+    if(ponpoint && [(GRPathEditor *)[path editor] isDone] && (self == mtopoint))
         [ponpoint moveBezierHandleToPosition: newp oldPosition: oldp];
 
     if(pointInRect(bzHandle.firstHandleRect, oldp)) {
@@ -136,19 +136,71 @@
     bzHandle.secondHandleRect = NSMakeRect(bzHandle.secondHandle.x-2, bzHandle.secondHandle.y-2, 4, 4);
 }
 
+- (void)drawControlAsSelected: (BOOL)sel
+{
+  NSPoint centerZ;
+  NSRect centerRectZ;
+  NSRect innerRectZ;
+
+  centerZ.x = bzHandle.center.x * zmFactor;
+  centerZ.y = bzHandle.center.y * zmFactor;
+  centerRectZ = NSMakeRect(centerZ.x-3, centerZ.y-3, 6, 6);
+  innerRectZ = NSMakeRect(centerZ.x-2, centerZ.y-2, 4, 4);
+ 
+  if (sel)
+    {
+      [[NSColor blackColor] set];
+      NSRectFill(centerRectZ);
+    }
+  else
+    {
+      [[NSColor whiteColor] set];
+      NSRectFill(centerRectZ);
+      [[NSColor blackColor] set];
+      NSFrameRect(centerRectZ);
+    }
+}
+
+- (void)drawHandle;
+{
+  NSPoint firstHandleP;
+  NSRect firstHandleR;
+  NSPoint secondHandleP;
+  NSRect secondHandleR;
+  NSPoint centerP;
+  NSBezierPath *bzp;
+
+  /* we calculate the zoomed coordinates */
+  firstHandleP = NSMakePoint(bzHandle.firstHandle.x * zmFactor, bzHandle.firstHandle.y * zmFactor);
+  secondHandleP = NSMakePoint(bzHandle.secondHandle.x * zmFactor, bzHandle.secondHandle.y * zmFactor);
+  centerP = NSMakePoint(bzHandle.center.x * zmFactor, bzHandle.center.y * zmFactor);
+
+  firstHandleR = NSMakeRect(firstHandleP.x-2, firstHandleP.y-2, 4, 4);
+  secondHandleR = NSMakeRect(secondHandleP.x-2, secondHandleP.y-2, 4, 4);
+  bzp = [NSBezierPath bezierPath];
+  
+  [bzp setLineWidth:1];
+  
+  [[NSColor blackColor] set];
+  NSRectFill(firstHandleR);
+  [bzp moveToPoint:firstHandleP];
+  [bzp lineToPoint:centerP];
+  [bzp lineToPoint:secondHandleP];
+  [bzp stroke];
+  NSRectFill(secondHandleR);
+}
+
+- (void)drawControl
+{
+  [self drawControlAsSelected:isSelect];
+  if (isSelect && isActiveHandle)
+    [self drawHandle];
+}
+
 - (void)setZoomFactor:(CGFloat)f
 {
-    bzHandle.center.x = bzHandle.center.x / zmFactor * f;
-    bzHandle.center.y = bzHandle.center.y / zmFactor * f;
-    bzHandle.centerRect = NSMakeRect(bzHandle.center.x-3, bzHandle.center.y-3, 6, 6);
-    bzHandle.firstHandle.x = bzHandle.firstHandle.x / zmFactor * f;
-    bzHandle.firstHandle.y = bzHandle.firstHandle.y / zmFactor * f;
-    bzHandle.firstHandleRect = NSMakeRect(bzHandle.firstHandle.x-2, bzHandle.firstHandle.y-2, 4, 4);
-    bzHandle.secondHandle.x = bzHandle.secondHandle.x / zmFactor * f;
-    bzHandle.secondHandle.y = bzHandle.secondHandle.y / zmFactor * f;
-    bzHandle.secondHandleRect = NSMakeRect(bzHandle.secondHandle.x-2, bzHandle.secondHandle.y-2, 4, 4);
 
-    zmFactor = f;
+  zmFactor = f;
 }
 
 - (GRBezierHandle)bzHandle
