@@ -61,12 +61,13 @@ typedef struct
 
 - (void)setZoomFactor:(CGFloat)f;
 
+- (BOOL)onControlPoint:(NSPoint)p;
 - (BOOL)onPathBorder:(NSPoint)p;
 
 - (GRBezierControlPoint *)firstPoint;
 - (GRBezierControlPoint *)lastPoint;
 
-- (int)indexOfPoint:(GRBezierControlPoint *)aPoint;
+- (NSUInteger)indexOfPoint:(GRBezierControlPoint *)aPoint;
 
 
 @end
