@@ -36,8 +36,9 @@
 
 #define UNDO_ACTION_OBJPROPS @"Change Object Properties"
 
-#define ZOOM_FACTORS 8
-float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
+#define ZOOM_FACTORS 13
+#define STD_ZOOM_INDEX 5
+float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.5, 3, 4, 6};
 
 @implementation GRDocView
 
@@ -67,7 +68,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
       shiftclick = NO;
       altclick = NO;
       ctrlclick = NO;
-      zIndex = 2;
+      zIndex = STD_ZOOM_INDEX;
       zFactor = zFactors[zIndex];
     }
   return self;
@@ -133,38 +134,6 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
 }
 
 
-- (NSArray *)usedFonts
-{
-  NSMutableArray *usedfonts;
-  NSString *fname;
-  id obj;
-  NSUInteger i, j;
-  BOOL exist;
-
-  usedfonts = [NSMutableArray arrayWithCapacity: 1];
-  for(i = 0; i < [objects count]; i++)
-    {
-        exist = NO;
-        obj = [objects objectAtIndex: i];
-        if([obj isKindOfClass: [GRText class]])
-        {
-            fname = [obj fontName];
-            for(j = 0; j < [usedfonts count]; j++)
-            {
-                if([[usedfonts objectAtIndex: j] isEqualToString: fname])
-                {
-                    exist = YES;
-                    break;
-                }
-            }
-            if(!exist)
-                [usedfonts addObject: fname];
-        }
-    }
-
-    return usedfonts;
-}
-
 - (BOOL)createObjectsFromDictionary:(NSDictionary *)dict
 {
     NSArray *keys;
@@ -427,7 +396,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
 		      pageRect.size.height-([pi topMargin]+[pi bottomMargin]));
     
   zmdRect = a4Rect;
-  zIndex = 2;
+  zIndex = STD_ZOOM_INDEX;
   zFactor = zFactors[zIndex];
                                                           
   [self setFrame: pageRect];
@@ -463,54 +432,58 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
 
 - (void)startDrawingAtPoint:(NSPoint)p
 {
-    NSEvent *nextEvent;
-    GRBezierPath *bzpath;
-    id obj;
-    BOOL isneweditor = YES;
-    int i;
-    NSUndoManager *uMgr;
-    
-    uMgr = [self undoManager];
-    /* save the method on the undo stack */
-    [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
-    [uMgr setActionName:@"Create Path"];
-    
-    [self saveCurrentObjects];
-
-    for(i = 0; i < [objects count]; i++)
+  NSEvent *nextEvent;
+  GRBezierPath *bzpath;
+  id obj;
+  BOOL isNewEditor = YES;
+  NSUInteger i;
+  NSUndoManager *uMgr;
+  
+  uMgr = [self undoManager];
+  /* save the method on the undo stack */
+  [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+  [uMgr setActionName:@"Create Path"];
+  
+  [self saveCurrentObjects];
+  
+  for(i = 0; i < [objects count]; i++)
     {
-        obj = [objects objectAtIndex: i];
-        if([obj isKindOfClass: [GRBezierPath class]])
-            if(![[obj editor] isdone])
-                isneweditor = NO;
+      obj = [objects objectAtIndex: i];
+      if([obj isKindOfClass: [GRBezierPath class]])
+        if(![[obj editor] isDone])
+          {
+            isNewEditor = NO;
+            edind = i;
+          }
     }
 
-    if(isneweditor)
-        for(i = 0; i < [objects count]; i++)
-        {
-            GRObjectEditor *objEdi;
-            
-            objEdi = [[objects objectAtIndex: i] editor];
-            if (![objEdi isSelect])
-                [objEdi unselect];
-        }
-
+  if(isNewEditor)
+    for(i = 0; i < [objects count]; i++)
+      {
+        GRObjectEditor *objEdi;
+        
+        objEdi = [[objects objectAtIndex: i] editor];
+        if (![objEdi isSelected])
+          [objEdi unselect];
+      }
+  
     nextEvent = [[self window] nextEventMatchingMask:
         NSLeftMouseUpMask | NSLeftMouseDraggedMask];
     [self verifyModifiersOfEvent: nextEvent];
 
     if([nextEvent type] != NSLeftMouseDragged)
-    {
-        if(isneweditor)
-        {
+      {
+        if(isNewEditor)
+          {
             [self addPath];
             bzpath = [objects objectAtIndex: edind];
             [[bzpath editor] selectForEditing];
             [bzpath addControlAtPoint: p];
             [self setNeedsDisplay: YES];
             return;
-        } else
-        {
+          }
+        else
+          {
             bzpath = [objects objectAtIndex: edind];
             [[bzpath editor] selectForEditing];
             if(shiftclick)
@@ -518,31 +491,35 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
             [bzpath addLineToPoint: p];
             [self setNeedsDisplay: YES];
             return;
-        }
-    } else
-    {
-        if(isneweditor)
-        {
+          }
+      }
+    else
+      {
+        if(isNewEditor)
+          {
             [self addPath];
             bzpath = [objects objectAtIndex: edind];
             [[bzpath editor] selectForEditing];
             [bzpath addControlAtPoint: p];
-        } else
-        {
+          }
+        else
+          {
             bzpath = [objects objectAtIndex: edind];
             [[bzpath editor] selectForEditing];
             if(shiftclick)
                 p = pointApplyingCostrainerToPoint(p, [[bzpath lastPoint] center]);
             [bzpath addControlAtPoint: p];
-        }
+          }
         [self setNeedsDisplay: YES];
-
+        
         do
-        {
+          {
             p = [nextEvent locationInWindow];
             p = [self convertPoint: p fromView: nil];
+	    p = GRpointDeZoom(p, zFactor);
+
             if(shiftclick)
-                p = pointApplyingCostrainerToPoint(p, [[bzpath lastPoint] center]);
+              p = pointApplyingCostrainerToPoint(p, [[bzpath lastPoint] center]);
 
             [bzpath addCurveWithBezierHandlePosition: p];
 
@@ -551,11 +528,12 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
             nextEvent = [[self window] nextEventMatchingMask:
                 NSLeftMouseUpMask | NSLeftMouseDraggedMask];
             [self verifyModifiersOfEvent: nextEvent];
-        } while([nextEvent type] != NSLeftMouseUp);
-
+          }
+        while([nextEvent type] != NSLeftMouseUp);
+        
         [bzpath confirmNewCurve];
         [self setNeedsDisplay: YES];
-    }
+      }
 }
 
 - (void)startBoxAtPoint:(NSPoint)p
@@ -577,7 +555,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
       GRObjectEditor *objEdi;
             
       objEdi = [[objects objectAtIndex: i] editor];
-      if (![objEdi isSelect])
+      if (![objEdi isSelected])
         [objEdi unselect];
     }
 
@@ -635,7 +613,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
       GRObjectEditor *objEdi;
 
       objEdi = [[objects objectAtIndex: i] editor];
-      if (![objEdi isSelect])
+      if (![objEdi isSelected])
         [objEdi unselect];
     }
 
@@ -680,85 +658,115 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
   id obj;
   NSMutableArray *objs;
   NSUInteger i;
+  GRDrawableObject *hitObj;
 
   objs = [NSMutableArray arrayWithCapacity: 1];
 
+  hitObj = nil;
   for(i = 0; i < [objects count]; i++)
     {
       obj = [objects objectAtIndex: i];
 
-      if([obj objectHitForSelection: p])
-        {
-          if (shiftclick && [[obj editor] isSelect])
-            [[obj editor] unselect];
-          else
-            [[obj editor] select];
-        }
-      else if(!shiftclick)
-	[[obj editor] unselect];
+      if ([obj objectHitForSelection: p])
+        hitObj = obj;
+      else if ([[obj editor] isSelected])
+        [objs addObject: obj];
     }
 
-  for(i = 0; i < [objects count]; i++)
+  /* with shift we add or remove the hit object from the list*/
+  if (shiftclick && hitObj)
     {
-        obj = [objects objectAtIndex: i];
-        if([[obj editor] isGroupSelected])
-            [objs addObject: obj];
+      if ([[hitObj editor] isSelected])
+        {
+          [[hitObj editor] unselect];
+        }
+      else
+        {
+          [[hitObj editor] select];
+          [objs addObject:hitObj];
+        }
     }
-
-    [self setNeedsDisplay: YES];
-
-    if([objs count])
-        [self moveSelectedObjects: objs startingPoint: p];
-    [[NSNotificationCenter defaultCenter] postNotificationName:@"ObjectSelectionChanged" object:self];
+  else
+    {
+      /* if not shift, if the object is already selected
+         we essentially don't change the selection.
+         else, we select it and deselect the rest */
+      if ([[hitObj editor] isSelected])
+        {
+          [objs addObject:hitObj];
+        }
+      else
+        {
+          NSEnumerator *e;
+          GRDrawableObject *o;
+          
+          e = [objs objectEnumerator];
+          while ((o = [e nextObject]))
+            {
+              [[o editor] unselect];
+            }
+          [objs removeAllObjects];
+          if (hitObj)
+            {
+              [[hitObj editor] select];
+              [objs addObject: hitObj];
+            }
+        }
+    }
+  [self setNeedsDisplay: YES];
+  
+  if([objs count])
+    [self moveSelectedObjects: objs startingPoint: p];
+  [[NSNotificationCenter defaultCenter] postNotificationName:@"ObjectSelectionChanged" object:self];
 }
 
 - (void)editPathAtPoint:(NSPoint)p
 {
-    id obj;
-    NSUInteger i;
-    NSUndoManager *uMgr;
-    
-    uMgr = [self undoManager];
-    /* save the method on the undo stack */
-    [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
-    [uMgr setActionName:@"Edit Path"];
-    
-    [self saveCurrentObjectsDeep];
+  id obj;
+  NSUInteger i;
+  NSUndoManager *uMgr;
+  
+  uMgr = [self undoManager];
+  /* save the method on the undo stack */
+  [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+  [uMgr setActionName:@"Edit Path"];
+  
+  [self saveCurrentObjects];
 
-    for(i = 0; i < [objects count]; i++)
+  /* we look for a path that is selected and process editing
+     we no longer auto-select the object for editing though, nor automatically unselect the editing of the object */
+  for(i = 0; i < [objects count]; i++)
     {
-        obj = [objects objectAtIndex: i];
-        if([obj isKindOfClass: [GRBezierPath class]])
+      obj = [objects objectAtIndex: i];
+      if([obj isKindOfClass: [GRBezierPath class]] && [[obj editor] isSelected])
         {
-            if([obj onPathBorder: p])
+          if([obj onControlPoint: p])
             {
-                [[obj editor] selectForEditing];
-                [self moveControlPointOfEditor: (GRBezierPathEditor *)[obj editor] toPoint: p];
-                return;
-            } else
-            {
-                if([self moveBezierHandleOfEditor: (GRBezierPathEditor *)[obj editor] toPoint: p])
-                    return;
-                else
-                    [[obj editor] unselect];
+              [[obj editor] selectForEditing];
+              [self moveControlPointOfEditor: (GRBezierPathEditor *)[obj editor] toPoint: p];
+              return;
             }
-        } else if ([obj isKindOfClass: [GRBox class]] || [obj isKindOfClass: [GRCircle class]])
-        {
-            if([obj onControlPoint: p])
+          else
             {
-                [[obj editor] selectForEditing];
-                [self moveControlPointOfEditor: (GRBezierPathEditor *)[obj editor] toPoint: p];
+              if([self moveBezierHandleOfEditor: (GRBezierPathEditor *)[obj editor] toPoint: p])
                 return;
-            } else
-            {
-                [[obj editor] unselect];
             }
-        } else
+        }
+      else if ([[obj editor] isSelected] && ([obj isKindOfClass: [GRBox class]] || [obj isKindOfClass: [GRCircle class]]))
         {
-            [[obj editor] unselect];
+          if([obj onControlPoint: p])
+            {
+              [[obj editor] selectForEditing];
+              [self moveControlPointOfEditor: (GRBezierPathEditor *)[obj editor] toPoint: p];
+              return;
+            }
+        }
+      else if([obj isKindOfClass: [GRText class]] && [[obj editor] isSelected])
+        {
+          /* we have no actions for GRText */
         }
     }
-    [self setNeedsDisplay: YES];
+  [self setNeedsDisplay: YES];
 }
 
 - (void)editTextAtPoint:(NSPoint)p
@@ -828,80 +836,85 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
     [self saveCurrentObjectsDeep];
 
     nextEvent = [[self window] nextEventMatchingMask:
-        NSLeftMouseUpMask | NSLeftMouseDraggedMask];
+				 NSLeftMouseUpMask | NSLeftMouseDraggedMask];
     if([nextEvent type] == NSLeftMouseDragged)
-    {
+      {
         [self verifyModifiersOfEvent: nextEvent];
         op.x = startp.x;
         op.y = startp.y;
-
+	
         do
-        {
-            p = [nextEvent locationInWindow];
-            p = [self convertPoint: p fromView: nil];
-            if(shiftclick)
-                p = pointApplyingCostrainerToPoint(p, startp);
-
-            if(altclick && !dupl)
-            {
-                moveobjs = [self duplicateObjects: objs andMoveTo: NSMakePoint(0, 0)];
-                dupl = YES;
-            } else if(!moveobjs)
-            {
-                moveobjs = [NSArray arrayWithArray: objs];
-            }
-
-            diffp.x = p.x - op.x;
-            diffp.y = p.y - op.y;
-            for(i = 0; i < [moveobjs count]; i++)
-            {
-                obj = [moveobjs objectAtIndex: i];
-                [obj moveAddingCoordsOfPoint: diffp];
-            }
-            op.x = p.x;
-            op.y = p.y;
-            [self setNeedsDisplay: YES];
-            nextEvent = [[self window] nextEventMatchingMask:
-                NSLeftMouseUpMask | NSLeftMouseDraggedMask];
-            [self verifyModifiersOfEvent: nextEvent];
-        } while([nextEvent type] != NSLeftMouseUp);
-
+	  {
+	    p = [nextEvent locationInWindow];
+	    p = [self convertPoint: p fromView: nil];
+	    p = GRpointDeZoom(p, zFactor);
+	    
+	    if(shiftclick)
+	      p = pointApplyingCostrainerToPoint(p, startp);
+	    
+	    if(altclick && !dupl)
+	      {
+		moveobjs = [self duplicateObjects: objs andMoveTo: NSMakePoint(0, 0)];
+		dupl = YES;
+	      }
+	    else if(!moveobjs)
+	      {
+		moveobjs = [NSArray arrayWithArray: objs];
+	      }
+	    
+	    diffp.x = p.x - op.x;
+	    diffp.y = p.y - op.y;
+	    for(i = 0; i < [moveobjs count]; i++)
+	      {
+		obj = [moveobjs objectAtIndex: i];
+		[obj moveAddingCoordsOfPoint: diffp];
+	      }
+	    op.x = p.x;
+	    op.y = p.y;
+	    [self setNeedsDisplay: YES];
+	    nextEvent = [[self window] nextEventMatchingMask:
+					 NSLeftMouseUpMask | NSLeftMouseDraggedMask];
+	    [self verifyModifiersOfEvent: nextEvent];
+	  }
+	while([nextEvent type] != NSLeftMouseUp);
+	
         if(dupl)
-        {
+	  {
             diffp.x = p.x - startp.x;
             diffp.y = p.y - startp.y;
-        } else
-        {
+	  }
+	else
+	  {
             diffp.x = startp.x - p.x;
             diffp.y = startp.y - p.y;
-        }
-    }
+	  }
+      }
 }
 
 
 /* propagates control point editing down to each editor */
 - (BOOL)moveControlPointOfEditor:(GRPathEditor *)editor toPoint:(NSPoint)pos
 {
-    NSPoint p;
+  NSPoint p;
 
-    p = [editor moveControlAtPoint: pos];
-    if(p.x == pos.x && p.y == pos.y)
-        return NO;
-
-    [self setNeedsDisplay: YES];
-    return YES;
+  p = [editor moveControlAtPoint: pos];
+  if(p.x == pos.x && p.y == pos.y)
+    return NO;
+  
+  [self setNeedsDisplay: YES];
+  return YES;
 }
 
 - (BOOL)moveBezierHandleOfEditor:(GRBezierPathEditor *)editor toPoint:(NSPoint)pos
 {
-    NSPoint p;
+  NSPoint p;
 
-    p = [editor moveBezierHandleAtPoint: pos];
-    if(p.x == pos.x && p.y == pos.y)
-        return NO;
-
-    [self setNeedsDisplay: YES];
-    return YES;
+  p = [editor moveBezierHandleAtPoint: pos];
+  if(p.x == pos.x && p.y == pos.y)
+    return NO;
+  
+  [self setNeedsDisplay: YES];
+  return YES;
 }
 
 - (void)subdividePathAtPoint:(NSPoint)p splitIt:(BOOL)split
@@ -945,7 +958,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
       id obj;
       obj = [objects objectAtIndex: i];
       
-      if([[obj editor] isSelect])
+      if([[obj editor] isSelected])
         {
           selectedObjects++;
         
@@ -1018,7 +1031,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
   for(i = 0; i < [objects count]; i++)
     {
       obj = [objects objectAtIndex: i];
-      if([[obj editor] isSelect])
+      if([[obj editor] isSelected])
         {
           NSColor *newColor;
       
@@ -1047,40 +1060,39 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
   NSUInteger i;
   NSUndoManager *uMgr;
 
-    uMgr = [self undoManager];
-    /* save the method on the undo stack */
-    [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
-    [uMgr setActionName:@"Move to front"];
-
-    [self saveCurrentObjectsDeep];
-
-
-    for(i = 0; i < [objects count]; i++)
+  uMgr = [self undoManager];
+  /* save the method on the undo stack */
+  [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+  [uMgr setActionName:@"Move to front"];
+  
+  [self saveCurrentObjectsDeep];
+  
+  for(i = 0; i < [objects count]; i++)
     {
-        if([[[objects objectAtIndex: i] editor] isGroupSelected])
+      if([[[objects objectAtIndex: i] editor] isGroupSelected])
         {
-            obj = [[objects objectAtIndex: i] retain];
-            break;
+          obj = [[objects objectAtIndex: i] retain];
+          break;
         }
     }
-    if(!obj)
-        return;
+  if(!obj)
+    return;
 
-    for(i = 0; i < [objects count]; i++)
-        if([objects objectAtIndex: i] != obj)
-            [[[objects objectAtIndex: i] editor] unselect];
-
-    for(i = 0; i < [objects count]; i++)
+  for(i = 0; i < [objects count]; i++)
+    if([objects objectAtIndex: i] != obj)
+      [[[objects objectAtIndex: i] editor] unselect];
+  
+  for(i = 0; i < [objects count]; i++)
     {
-        if((obj == [objects objectAtIndex: i]) && (i + 1 < [objects count]))
+      if((obj == [objects objectAtIndex: i]) && (i + 1 < [objects count]))
         {
-            [objects removeObjectAtIndex: i];
-            [objects insertObject: obj atIndex: i + 1];
-            break;
+          [objects removeObjectAtIndex: i];
+          [objects insertObject: obj atIndex: i + 1];
+          break;
         }
     }
-    [obj release];
-    [self setNeedsDisplay: YES];
+  [obj release];
+  [self setNeedsDisplay: YES];
 }
 
 - (void)moveSelectedObjectsToBack:(id)sender
@@ -1089,57 +1101,57 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
   NSUInteger i;
   NSUndoManager *uMgr;
 
-    uMgr = [self undoManager];
-    /* save the method on the undo stack */
-    [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
-    [uMgr setActionName:@"Move to back"];
-
-    [self saveCurrentObjects];
-    
-    for(i = 0; i < [objects count]; i++)
+  uMgr = [self undoManager];
+  /* save the method on the undo stack */
+  [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+  [uMgr setActionName:@"Move to back"];
+  
+  [self saveCurrentObjects];
+  
+  for(i = 0; i < [objects count]; i++)
     {
-        if([[[objects objectAtIndex: i] editor] isGroupSelected])
+      if([[[objects objectAtIndex: i] editor] isGroupSelected])
         {
-            obj = [[objects objectAtIndex: i] retain];
-            break;
+          obj = [[objects objectAtIndex: i] retain];
+          break;
         }
     }
-    if(!obj)
-        return;
-
-    for(i = 0; i < [objects count]; i++)
-        if([objects objectAtIndex: i] != obj)
-            [[[objects objectAtIndex: i] editor] unselect];
-
-    for(i = 0; i < [objects count]; i++)
+  if(!obj)
+    return;
+  
+  for(i = 0; i < [objects count]; i++)
+    if([objects objectAtIndex: i] != obj)
+      [[[objects objectAtIndex: i] editor] unselect];
+  
+  for(i = 0; i < [objects count]; i++)
     {
-        if((obj == [objects objectAtIndex: i]) && (i > 0)) 
+      if((obj == [objects objectAtIndex: i]) && (i > 0)) 
         {
-            [objects removeObjectAtIndex: i];
-            [objects insertObject: obj atIndex: i - 1];
-            break;
+          [objects removeObjectAtIndex: i];
+          [objects insertObject: obj atIndex: i - 1];
+          break;
         }
     }
-    [obj release];
-    [self setNeedsDisplay: YES];
+  [obj release];
+  [self setNeedsDisplay: YES];
 }
 
 - (void)unselectOtherObjects:(GRDrawableObject *)anObject
 {
-    id obj;
-    int i;
+  id obj;
+  NSUInteger i;
 
-    if(shiftclick)
-        return;
-
-    for(i = 0; i < [objects count]; i++)
+  if(shiftclick)
+    return;
+  
+  for(i = 0; i < [objects count]; i++)
     {
-        obj = [objects objectAtIndex: i];
-        if(obj != anObject)
-            [[obj editor] unselect];
+      obj = [objects objectAtIndex: i];
+      if(obj != anObject)
+        [[obj editor] unselect];
     }
-
-    [self setNeedsDisplay: YES];
+  
+  [self setNeedsDisplay: YES];
 }
 
 - (void)zoomOnPoint:(NSPoint)p zoomOut:(BOOL)isout
@@ -1256,13 +1268,15 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
   NSUInteger i;
   NSPoint p;
   NSRect visibleRect;
-  NSRect f;
+  NSRect docVRect;
+  CGFloat fWidth;
+  CGFloat fHeight;
 
-  f = [self frame];
-  visibleRect = [self visibleRect];
+  fWidth = [self frame].size.width / zFactor;
+  fHeight = [self frame].size.height / zFactor;
+  docVRect = [(NSClipView*)[self superview] documentVisibleRect];
   i = ZOOM_FACTORS - 1;
-  NSLog(@"visible %f, full %f", visibleRect.size.width, f.size.width);
-  while (i > 0 && ((visibleRect.size.width < f.size.width * zFactors[i]) || (visibleRect.size.height < f.size.height * zFactors[i])))
+  while (i > 0 && ((docVRect.size.width < fWidth * zFactors[i]) || (docVRect.size.height < fHeight * zFactors[i])))
     i--;
 
   visibleRect = [self visibleRect];
@@ -1277,13 +1291,13 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
   NSUInteger i;
   NSPoint p;
   NSRect visibleRect;
-  NSRect f;
+  NSRect docVRect;
+  CGFloat fWidth;
 
-  f = [self frame];
-  visibleRect = [self visibleRect];
+  fWidth = [self frame].size.width / zFactor;
+  docVRect = [(NSClipView*)[self superview] documentVisibleRect];
   i = ZOOM_FACTORS - 1;
-  NSLog(@"visible %f, full %f", visibleRect.size.width, f.size.width);
-  while (i > 0 && visibleRect.size.width < f.size.width * zFactors[i])
+  while (i > 0 && docVRect.size.width < fWidth * zFactors[i])
     i--;
  
   visibleRect = [self visibleRect];
@@ -1295,27 +1309,31 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
 
 - (void)movePageFromHandPoint:(NSPoint)handpos
 {
-    NSEvent *nextEvent;
-    NSPoint p, diffp;
-    NSRect r;
-
-    nextEvent = [[self window] nextEventMatchingMask:
-        NSLeftMouseUpMask | NSLeftMouseDraggedMask];
-    if([nextEvent type] == NSLeftMouseDragged)
+  NSEvent *nextEvent;
+  NSPoint p, diffp;
+  NSRect r;
+  
+  nextEvent = [[self window] nextEventMatchingMask:
+                               NSLeftMouseUpMask | NSLeftMouseDraggedMask];
+  if([nextEvent type] == NSLeftMouseDragged)
     {
-        do {
-            p = [nextEvent locationInWindow];
-            p = [self convertPoint: p fromView: nil];
-            diffp.x = p.x - handpos.x;
-            diffp.y = p.y - handpos.y;
-            r = [self visibleRect];
-            r = NSMakeRect(r.origin.x - diffp.x, r.origin.y - diffp.y,
-                           r.size.width, r.size.height);
-            [self scrollRectToVisible: r];
-            nextEvent = [[self window] nextEventMatchingMask:
-                NSLeftMouseUpMask | NSLeftMouseDraggedMask];
-        } while([nextEvent type] != NSLeftMouseUp);
-        [[self window] display];
+      do
+        {
+          p = [nextEvent locationInWindow];
+          p = [self convertPoint: p fromView: nil];
+          p = GRpointDeZoom(p, zFactor);
+          
+          diffp.x = p.x - handpos.x;
+          diffp.y = p.y - handpos.y;
+          r = [self visibleRect];
+          r = NSMakeRect(r.origin.x - diffp.x, r.origin.y - diffp.y,
+                         r.size.width, r.size.height);
+          [self scrollRectToVisible: r];
+          nextEvent = [[self window] nextEventMatchingMask:
+                                       NSLeftMouseUpMask | NSLeftMouseDraggedMask];
+        }
+      while([nextEvent type] != NSLeftMouseUp);
+      [[self window] display];
     }
 }
 
@@ -1497,7 +1515,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
             shiftclick = YES;
         else
             shiftclick = NO;
-        if([theEvent modifierFlags] & NSCommandKeyMask)
+        if([theEvent modifierFlags] & NSAlternateKeyMask)
             altclick = YES;
         else
             altclick = NO;
@@ -1507,11 +1525,11 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
             ctrlclick = NO;
     }
 
-    if([theEvent type] == NSLeftMouseUp)
+    else if([theEvent type] == NSLeftMouseUp)
     {
         if(!([theEvent modifierFlags] & NSShiftKeyMask))
             shiftclick = NO;
-        if(!([theEvent modifierFlags] & NSCommandKeyMask))
+        if(!([theEvent modifierFlags] & NSAlternateKeyMask))
             altclick = NO;
         if(!([theEvent modifierFlags] & NSControlKeyMask))
             ctrlclick = NO;
@@ -1542,6 +1560,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
 
   p = [theEvent locationInWindow];
   p = [self convertPoint: p fromView: nil];
+  p = GRpointDeZoom(p, zFactor);
 
   if(count == 1)
     {
@@ -1598,10 +1617,11 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
             default:
                 break;
         }
-    } else
+    }
+  else
     {
-        if([[NSApp delegate] currentToolType] == blackarrowtool)
-            [self editTextAtPoint: p];
+      if([[NSApp delegate] currentToolType] == blackarrowtool)
+	[self editTextAtPoint: p];
     }
 }
 
