@@ -2,7 +2,7 @@
  Project: Graphos
  GRObjectEditor.m
 
- Copyright (C) 2008-2010 GNUstep Application Project
+ Copyright (C) 2008-2013 GNUstep Application Project
 
  Author: Ing. Riccardo Mottola
 
@@ -37,8 +37,8 @@
         object = anObject;
         groupSelected = NO;
         editSelected = NO;
-        isdone = NO;
-        isvalid = NO;
+        isDone = NO;
+        isValid = NO;
     }
     return self;
 }
@@ -73,7 +73,7 @@
     {
         groupSelected = YES;
         editSelected = NO;
-        isvalid = NO;
+        isValid = NO;
         
 	[[object view] unselectOtherObjects: (GRDrawableObject *)object];
     }
@@ -85,7 +85,8 @@
         return;
     editSelected = YES;
     groupSelected = NO;
-    isvalid = NO;
+    isValid = NO;
+    isDone = NO;
     [[object view] unselectOtherObjects: (GRDrawableObject *)object];
 }
 
@@ -94,11 +95,11 @@
 {
     groupSelected = NO;
     editSelected = NO;
-    isvalid = YES;
-    isdone = YES;
+    isValid = YES;
+    isDone = YES;
 }
 
-- (BOOL)isSelect
+- (BOOL)isSelected
 {
     if(editSelected || groupSelected)
         return YES;
@@ -115,14 +116,25 @@
     return editSelected;
 }
 
-- (BOOL)isdone
+- (BOOL)isDone
 {
-    return isdone;
+  return isDone;
 }
 
 - (void)setIsDone:(BOOL)status
 {
-    isdone = status;
+  isDone = status;
+}
+
+
+- (void)setIsValid:(BOOL)value
+{
+  isValid = value;
+}
+
+- (BOOL)isValid
+{
+  return isValid;
 }
 
 - (void)draw
